/* -*- c-file-style: "xxdiff" -*- */
/******************************************************************************\
 * $RCSfile$
 *
 * Copyright (C) 1999-2003  Martin Blais <blais@furius.ca>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *****************************************************************************/

/*==============================================================================
 * EXTERNAL DECLARATIONS
 *============================================================================*/

#include <proginfo.h>
#include <version.h>


/*==============================================================================
 * PUBLIC DECLARATIONS
 *============================================================================*/

char const xx_name[] = "xxdiff";
char const xx_version[] = XX_VERSION;
char const xx_description[] = "A graphical file comparator and merge tool.";
char const xx_copyright[] = "Copyright (C) 1999-2004  Martin Blais";
char const xx_homepage[] = "http://github.com/blais/xxdiff";
char const xx_bugs_email[] = "blais@furius.ca";
