/****************************************************************************
** Meta object code from reading C++ file 'optionsDialog.h'
**
** Created by: The Qt Meta Object Compiler version 68 (Qt 6.7.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "optionsDialog.h"
#include <QtGui/qtextcursor.h>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'optionsDialog.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 68
#error "This file was generated using the moc from 6.7.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {

#ifdef QT_MOC_HAS_STRINGDATA
struct qt_meta_stringdata_CLASSXxOptionsDialogENDCLASS_t {};
constexpr auto qt_meta_stringdata_CLASSXxOptionsDialogENDCLASS = QtMocHelpers::stringData(
    "XxOptionsDialog",
    "synchronize",
    "",
    "onApply",
    "onClose",
    "checkboxIgnoreTrailingBlanks",
    "checkboxIgnoreWhitespace",
    "checkboxIgnoreCase",
    "checkboxIgnoreBlankLines",
    "radiobuttonQualityNormal",
    "radiobuttonQualityFastest",
    "radiobuttonQualityHighest",
    "editFontApp",
    "editFontText",
    "listboxColors",
    "QListWidgetItem*",
    "editColorFore",
    "editColorBack"
);
#else  // !QT_MOC_HAS_STRINGDATA
#error "qtmochelpers.h not found or too old."
#endif // !QT_MOC_HAS_STRINGDATA
} // unnamed namespace

Q_CONSTINIT static const uint qt_meta_data_CLASSXxOptionsDialogENDCLASS[] = {

 // content:
      12,       // revision
       0,       // classname
       0,    0, // classinfo
      15,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags, initial metatype offsets
       1,    0,  104,    2, 0x0a,    1 /* Public */,
       3,    0,  105,    2, 0x0a,    2 /* Public */,
       4,    0,  106,    2, 0x0a,    3 /* Public */,
       5,    1,  107,    2, 0x0a,    4 /* Public */,
       6,    1,  110,    2, 0x0a,    6 /* Public */,
       7,    1,  113,    2, 0x0a,    8 /* Public */,
       8,    1,  116,    2, 0x0a,   10 /* Public */,
       9,    1,  119,    2, 0x0a,   12 /* Public */,
      10,    1,  122,    2, 0x0a,   14 /* Public */,
      11,    1,  125,    2, 0x0a,   16 /* Public */,
      12,    0,  128,    2, 0x0a,   18 /* Public */,
      13,    0,  129,    2, 0x0a,   19 /* Public */,
      14,    2,  130,    2, 0x0a,   20 /* Public */,
      16,    0,  135,    2, 0x0a,   23 /* Public */,
      17,    0,  136,    2, 0x0a,   24 /* Public */,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 15, 0x80000000 | 15,    2,    2,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

Q_CONSTINIT const QMetaObject XxOptionsDialog::staticMetaObject = { {
    QMetaObject::SuperData::link<QDialog::staticMetaObject>(),
    qt_meta_stringdata_CLASSXxOptionsDialogENDCLASS.offsetsAndSizes,
    qt_meta_data_CLASSXxOptionsDialogENDCLASS,
    qt_static_metacall,
    nullptr,
    qt_incomplete_metaTypeArray<qt_meta_stringdata_CLASSXxOptionsDialogENDCLASS_t,
        // Q_OBJECT / Q_GADGET
        QtPrivate::TypeAndForceComplete<XxOptionsDialog, std::true_type>,
        // method 'synchronize'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'onApply'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'onClose'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'checkboxIgnoreTrailingBlanks'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<int, std::false_type>,
        // method 'checkboxIgnoreWhitespace'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<int, std::false_type>,
        // method 'checkboxIgnoreCase'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<int, std::false_type>,
        // method 'checkboxIgnoreBlankLines'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<int, std::false_type>,
        // method 'radiobuttonQualityNormal'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<bool, std::false_type>,
        // method 'radiobuttonQualityFastest'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<bool, std::false_type>,
        // method 'radiobuttonQualityHighest'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<bool, std::false_type>,
        // method 'editFontApp'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'editFontText'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'listboxColors'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<QListWidgetItem *, std::false_type>,
        QtPrivate::TypeAndForceComplete<QListWidgetItem *, std::false_type>,
        // method 'editColorFore'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'editColorBack'
        QtPrivate::TypeAndForceComplete<void, std::false_type>
    >,
    nullptr
} };

void XxOptionsDialog::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<XxOptionsDialog *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->synchronize(); break;
        case 1: _t->onApply(); break;
        case 2: _t->onClose(); break;
        case 3: _t->checkboxIgnoreTrailingBlanks((*reinterpret_cast< std::add_pointer_t<int>>(_a[1]))); break;
        case 4: _t->checkboxIgnoreWhitespace((*reinterpret_cast< std::add_pointer_t<int>>(_a[1]))); break;
        case 5: _t->checkboxIgnoreCase((*reinterpret_cast< std::add_pointer_t<int>>(_a[1]))); break;
        case 6: _t->checkboxIgnoreBlankLines((*reinterpret_cast< std::add_pointer_t<int>>(_a[1]))); break;
        case 7: _t->radiobuttonQualityNormal((*reinterpret_cast< std::add_pointer_t<bool>>(_a[1]))); break;
        case 8: _t->radiobuttonQualityFastest((*reinterpret_cast< std::add_pointer_t<bool>>(_a[1]))); break;
        case 9: _t->radiobuttonQualityHighest((*reinterpret_cast< std::add_pointer_t<bool>>(_a[1]))); break;
        case 10: _t->editFontApp(); break;
        case 11: _t->editFontText(); break;
        case 12: _t->listboxColors((*reinterpret_cast< std::add_pointer_t<QListWidgetItem*>>(_a[1])),(*reinterpret_cast< std::add_pointer_t<QListWidgetItem*>>(_a[2]))); break;
        case 13: _t->editColorFore(); break;
        case 14: _t->editColorBack(); break;
        default: ;
        }
    }
}

const QMetaObject *XxOptionsDialog::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *XxOptionsDialog::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_CLASSXxOptionsDialogENDCLASS.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "Ui::XxOptionsDialogBase"))
        return static_cast< Ui::XxOptionsDialogBase*>(this);
    return QDialog::qt_metacast(_clname);
}

int XxOptionsDialog::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDialog::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 15)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 15;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 15)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 15;
    }
    return _id;
}
QT_WARNING_POP
