/********************************************************************************
** Form generated from reading UI file 'searchDialogBase.ui'
**
** Created by: Qt User Interface Compiler version 6.7.2
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_SEARCHDIALOGBASE_H
#define UI_SEARCHDIALOGBASE_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>

QT_BEGIN_NAMESPACE

class Ui_XxSearchDialogBase
{
public:
    QVBoxLayout *vboxLayout;
    QHBoxLayout *hboxLayout;
    QLabel *TextLabel1;
    QLineEdit *_lineeditSearchString;
    QGroupBox *GroupBox1;
    QHBoxLayout *hboxLayout1;
    QCheckBox *_checkboxCaseSensitive;
    QCheckBox *_checkboxUseRegexp;
    QHBoxLayout *hboxLayout2;
    QPushButton *_buttonApply;
    QPushButton *_buttonPrevious;
    QPushButton *_buttonNext;
    QSpacerItem *Horizontal_Spacing2;
    QPushButton *_buttonClose;
    QHBoxLayout *hboxLayout3;
    QLabel *TextLabel1_2;
    QLineEdit *_lineeditGotoLine;
    QComboBox *_comboGotoWhichFile;
    QPushButton *_buttonGotoLine;
    QSpacerItem *Spacer2;

    void setupUi(QDialog *XxSearchDialogBase)
    {
        if (XxSearchDialogBase->objectName().isEmpty())
            XxSearchDialogBase->setObjectName("XxSearchDialogBase");
        XxSearchDialogBase->resize(619, 232);
        XxSearchDialogBase->setSizeGripEnabled(true);
        vboxLayout = new QVBoxLayout(XxSearchDialogBase);
#ifndef Q_OS_MAC
        vboxLayout->setSpacing(6);
#endif
        vboxLayout->setContentsMargins(11, 11, 11, 11);
        vboxLayout->setObjectName("vboxLayout");
        hboxLayout = new QHBoxLayout();
#ifndef Q_OS_MAC
        hboxLayout->setSpacing(6);
#endif
#ifndef Q_OS_MAC
        hboxLayout->setContentsMargins(0, 0, 0, 0);
#endif
        hboxLayout->setObjectName("hboxLayout");
        TextLabel1 = new QLabel(XxSearchDialogBase);
        TextLabel1->setObjectName("TextLabel1");
        TextLabel1->setWordWrap(false);

        hboxLayout->addWidget(TextLabel1);

        _lineeditSearchString = new QLineEdit(XxSearchDialogBase);
        _lineeditSearchString->setObjectName("_lineeditSearchString");

        hboxLayout->addWidget(_lineeditSearchString);


        vboxLayout->addLayout(hboxLayout);

        GroupBox1 = new QGroupBox(XxSearchDialogBase);
        GroupBox1->setObjectName("GroupBox1");
        hboxLayout1 = new QHBoxLayout(GroupBox1);
#ifndef Q_OS_MAC
        hboxLayout1->setSpacing(6);
#endif
        hboxLayout1->setContentsMargins(11, 11, 11, 11);
        hboxLayout1->setObjectName("hboxLayout1");
        _checkboxCaseSensitive = new QCheckBox(GroupBox1);
        _checkboxCaseSensitive->setObjectName("_checkboxCaseSensitive");
        _checkboxCaseSensitive->setEnabled(false);

        hboxLayout1->addWidget(_checkboxCaseSensitive);

        _checkboxUseRegexp = new QCheckBox(GroupBox1);
        _checkboxUseRegexp->setObjectName("_checkboxUseRegexp");
        _checkboxUseRegexp->setEnabled(false);

        hboxLayout1->addWidget(_checkboxUseRegexp);


        vboxLayout->addWidget(GroupBox1);

        hboxLayout2 = new QHBoxLayout();
#ifndef Q_OS_MAC
        hboxLayout2->setSpacing(6);
#endif
        hboxLayout2->setContentsMargins(0, 0, 0, 0);
        hboxLayout2->setObjectName("hboxLayout2");
        _buttonApply = new QPushButton(XxSearchDialogBase);
        _buttonApply->setObjectName("_buttonApply");
        _buttonApply->setAutoDefault(true);

        hboxLayout2->addWidget(_buttonApply);

        _buttonPrevious = new QPushButton(XxSearchDialogBase);
        _buttonPrevious->setObjectName("_buttonPrevious");

        hboxLayout2->addWidget(_buttonPrevious);

        _buttonNext = new QPushButton(XxSearchDialogBase);
        _buttonNext->setObjectName("_buttonNext");

        hboxLayout2->addWidget(_buttonNext);

        Horizontal_Spacing2 = new QSpacerItem(20, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        hboxLayout2->addItem(Horizontal_Spacing2);

        _buttonClose = new QPushButton(XxSearchDialogBase);
        _buttonClose->setObjectName("_buttonClose");
        _buttonClose->setAutoDefault(true);

        hboxLayout2->addWidget(_buttonClose);


        vboxLayout->addLayout(hboxLayout2);

        hboxLayout3 = new QHBoxLayout();
#ifndef Q_OS_MAC
        hboxLayout3->setSpacing(6);
#endif
        hboxLayout3->setContentsMargins(0, 0, 0, 0);
        hboxLayout3->setObjectName("hboxLayout3");
        TextLabel1_2 = new QLabel(XxSearchDialogBase);
        TextLabel1_2->setObjectName("TextLabel1_2");
        TextLabel1_2->setWordWrap(false);

        hboxLayout3->addWidget(TextLabel1_2);

        _lineeditGotoLine = new QLineEdit(XxSearchDialogBase);
        _lineeditGotoLine->setObjectName("_lineeditGotoLine");

        hboxLayout3->addWidget(_lineeditGotoLine);

        _comboGotoWhichFile = new QComboBox(XxSearchDialogBase);
        _comboGotoWhichFile->addItem(QString());
        _comboGotoWhichFile->addItem(QString());
        _comboGotoWhichFile->addItem(QString());
        _comboGotoWhichFile->setObjectName("_comboGotoWhichFile");

        hboxLayout3->addWidget(_comboGotoWhichFile);

        _buttonGotoLine = new QPushButton(XxSearchDialogBase);
        _buttonGotoLine->setObjectName("_buttonGotoLine");

        hboxLayout3->addWidget(_buttonGotoLine);

        Spacer2 = new QSpacerItem(20, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        hboxLayout3->addItem(Spacer2);


        vboxLayout->addLayout(hboxLayout3);


        retranslateUi(XxSearchDialogBase);
        QObject::connect(_buttonClose, &QPushButton::clicked, XxSearchDialogBase, qOverload<>(&QDialog::reject));

        _buttonApply->setDefault(true);


        QMetaObject::connectSlotsByName(XxSearchDialogBase);
    } // setupUi

    void retranslateUi(QDialog *XxSearchDialogBase)
    {
        XxSearchDialogBase->setWindowTitle(QCoreApplication::translate("XxSearchDialogBase", "Search", nullptr));
        TextLabel1->setText(QCoreApplication::translate("XxSearchDialogBase", "Search string:", nullptr));
        GroupBox1->setTitle(QCoreApplication::translate("XxSearchDialogBase", "Options", nullptr));
        _checkboxCaseSensitive->setText(QCoreApplication::translate("XxSearchDialogBase", "Case sensitive", nullptr));
        _checkboxUseRegexp->setText(QCoreApplication::translate("XxSearchDialogBase", "Regexp search", nullptr));
        _buttonApply->setText(QCoreApplication::translate("XxSearchDialogBase", "&Apply", nullptr));
        _buttonPrevious->setText(QCoreApplication::translate("XxSearchDialogBase", "<< Previous", nullptr));
        _buttonNext->setText(QCoreApplication::translate("XxSearchDialogBase", "Next >>", nullptr));
        _buttonClose->setText(QCoreApplication::translate("XxSearchDialogBase", "&Close", nullptr));
        TextLabel1_2->setText(QCoreApplication::translate("XxSearchDialogBase", "Goto line:", nullptr));
        _lineeditGotoLine->setText(QCoreApplication::translate("XxSearchDialogBase", "1", nullptr));
        _comboGotoWhichFile->setItemText(0, QCoreApplication::translate("XxSearchDialogBase", "left", nullptr));
        _comboGotoWhichFile->setItemText(1, QCoreApplication::translate("XxSearchDialogBase", "right", nullptr));
        _comboGotoWhichFile->setItemText(2, QCoreApplication::translate("XxSearchDialogBase", "middle", nullptr));

        _buttonGotoLine->setText(QCoreApplication::translate("XxSearchDialogBase", ">> Goto line", nullptr));
    } // retranslateUi

};

namespace Ui {
    class XxSearchDialogBase: public Ui_XxSearchDialogBase {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_SEARCHDIALOGBASE_H
