/********************************************************************************
** Form generated from reading UI file 'markersWidgetBase.ui'
**
** Created by: Qt User Interface Compiler version 6.7.2
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_MARKERSWIDGETBASE_H
#define UI_MARKERSWIDGETBASE_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QRadioButton>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_XxMarkersWidgetBase
{
public:
    QVBoxLayout *vboxLayout;
    QLabel *TextLabel1_2;
    QGroupBox *_groupbox;
    QVBoxLayout *vboxLayout1;
    QRadioButton *_checkboxConflicts;
    QHBoxLayout *hboxLayout;
    QRadioButton *_checkboxConditionals;
    QCheckBox *_removeEmptyConditionals;
    QHBoxLayout *hboxLayout1;
    QLabel *_labelConditional1;
    QLineEdit *_lineeditConditional1;
    QHBoxLayout *hboxLayout2;
    QLabel *_labelConditional2;
    QLineEdit *_lineeditConditional2;
    QHBoxLayout *hboxLayout3;
    QLabel *_labelConditional3;
    QLineEdit *_lineeditConditional3;

    void setupUi(QWidget *XxMarkersWidgetBase)
    {
        if (XxMarkersWidgetBase->objectName().isEmpty())
            XxMarkersWidgetBase->setObjectName("XxMarkersWidgetBase");
        XxMarkersWidgetBase->resize(537, 253);
        vboxLayout = new QVBoxLayout(XxMarkersWidgetBase);
#ifndef Q_OS_MAC
        vboxLayout->setSpacing(6);
#endif
        vboxLayout->setContentsMargins(11, 11, 11, 11);
        vboxLayout->setObjectName("vboxLayout");
        TextLabel1_2 = new QLabel(XxMarkersWidgetBase);
        TextLabel1_2->setObjectName("TextLabel1_2");
        QFont font;
        font.setFamilies({QString::fromUtf8("helvetica")});
        font.setPointSize(14);
        TextLabel1_2->setFont(font);
        TextLabel1_2->setAlignment(Qt::AlignCenter);
        TextLabel1_2->setWordWrap(false);

        vboxLayout->addWidget(TextLabel1_2);

        _groupbox = new QGroupBox(XxMarkersWidgetBase);
        _groupbox->setObjectName("_groupbox");
        vboxLayout1 = new QVBoxLayout(_groupbox);
#ifndef Q_OS_MAC
        vboxLayout1->setSpacing(6);
#endif
        vboxLayout1->setContentsMargins(11, 11, 11, 11);
        vboxLayout1->setObjectName("vboxLayout1");
        _checkboxConflicts = new QRadioButton(_groupbox);
        _checkboxConflicts->setObjectName("_checkboxConflicts");

        vboxLayout1->addWidget(_checkboxConflicts);

        hboxLayout = new QHBoxLayout();
#ifndef Q_OS_MAC
        hboxLayout->setSpacing(6);
#endif
        hboxLayout->setContentsMargins(0, 0, 0, 0);
        hboxLayout->setObjectName("hboxLayout");
        _checkboxConditionals = new QRadioButton(_groupbox);
        _checkboxConditionals->setObjectName("_checkboxConditionals");

        hboxLayout->addWidget(_checkboxConditionals);

        _removeEmptyConditionals = new QCheckBox(_groupbox);
        _removeEmptyConditionals->setObjectName("_removeEmptyConditionals");

        hboxLayout->addWidget(_removeEmptyConditionals);


        vboxLayout1->addLayout(hboxLayout);

        hboxLayout1 = new QHBoxLayout();
#ifndef Q_OS_MAC
        hboxLayout1->setSpacing(6);
#endif
        hboxLayout1->setContentsMargins(0, 0, 0, 0);
        hboxLayout1->setObjectName("hboxLayout1");
        _labelConditional1 = new QLabel(_groupbox);
        _labelConditional1->setObjectName("_labelConditional1");
        _labelConditional1->setWordWrap(false);

        hboxLayout1->addWidget(_labelConditional1);

        _lineeditConditional1 = new QLineEdit(_groupbox);
        _lineeditConditional1->setObjectName("_lineeditConditional1");

        hboxLayout1->addWidget(_lineeditConditional1);


        vboxLayout1->addLayout(hboxLayout1);

        hboxLayout2 = new QHBoxLayout();
#ifndef Q_OS_MAC
        hboxLayout2->setSpacing(6);
#endif
        hboxLayout2->setContentsMargins(0, 0, 0, 0);
        hboxLayout2->setObjectName("hboxLayout2");
        _labelConditional2 = new QLabel(_groupbox);
        _labelConditional2->setObjectName("_labelConditional2");
        _labelConditional2->setWordWrap(false);

        hboxLayout2->addWidget(_labelConditional2);

        _lineeditConditional2 = new QLineEdit(_groupbox);
        _lineeditConditional2->setObjectName("_lineeditConditional2");

        hboxLayout2->addWidget(_lineeditConditional2);


        vboxLayout1->addLayout(hboxLayout2);

        hboxLayout3 = new QHBoxLayout();
#ifndef Q_OS_MAC
        hboxLayout3->setSpacing(6);
#endif
        hboxLayout3->setContentsMargins(0, 0, 0, 0);
        hboxLayout3->setObjectName("hboxLayout3");
        _labelConditional3 = new QLabel(_groupbox);
        _labelConditional3->setObjectName("_labelConditional3");
        _labelConditional3->setWordWrap(false);

        hboxLayout3->addWidget(_labelConditional3);

        _lineeditConditional3 = new QLineEdit(_groupbox);
        _lineeditConditional3->setObjectName("_lineeditConditional3");

        hboxLayout3->addWidget(_lineeditConditional3);


        vboxLayout1->addLayout(hboxLayout3);


        vboxLayout->addWidget(_groupbox);


        retranslateUi(XxMarkersWidgetBase);

        QMetaObject::connectSlotsByName(XxMarkersWidgetBase);
    } // setupUi

    void retranslateUi(QWidget *XxMarkersWidgetBase)
    {
        XxMarkersWidgetBase->setWindowTitle(QCoreApplication::translate("XxMarkersWidgetBase", "Form3", nullptr));
        TextLabel1_2->setText(QCoreApplication::translate("XxMarkersWidgetBase", "Some regions were found that are still UNSELECTED.", nullptr));
        _groupbox->setTitle(QCoreApplication::translate("XxMarkersWidgetBase", "Unselected lines", nullptr));
        _checkboxConflicts->setText(QCoreApplication::translate("XxMarkersWidgetBase", "Save as merge conflicts", nullptr));
        _checkboxConditionals->setText(QCoreApplication::translate("XxMarkersWidgetBase", "Save with conditionals", nullptr));
        _removeEmptyConditionals->setText(QCoreApplication::translate("XxMarkersWidgetBase", "Remove empty conditionals", nullptr));
        _labelConditional1->setText(QCoreApplication::translate("XxMarkersWidgetBase", "Conditional variable:", nullptr));
        _lineeditConditional1->setText(QCoreApplication::translate("XxMarkersWidgetBase", "FILE1", nullptr));
        _labelConditional2->setText(QCoreApplication::translate("XxMarkersWidgetBase", "Conditional variable:", nullptr));
        _lineeditConditional2->setText(QCoreApplication::translate("XxMarkersWidgetBase", "FILE2", nullptr));
        _labelConditional3->setText(QCoreApplication::translate("XxMarkersWidgetBase", "Conditional variable:", nullptr));
        _lineeditConditional3->setText(QCoreApplication::translate("XxMarkersWidgetBase", "FILE3", nullptr));
    } // retranslateUi

};

namespace Ui {
    class XxMarkersWidgetBase: public Ui_XxMarkersWidgetBase {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_MARKERSWIDGETBASE_H
