/********************************************************************************
** Form generated from reading UI file 'optionsDialogBase.ui'
**
** Created by: Qt User Interface Compiler version 6.7.2
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_OPTIONSDIALOGBASE_H
#define UI_OPTIONSDIALOGBASE_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QListWidget>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QRadioButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QTabWidget>
#include <QtWidgets/QTextEdit>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_XxOptionsDialogBase
{
public:
    QVBoxLayout *vboxLayout;
    QTabWidget *_tabWidget;
    QWidget *Widget2;
    QVBoxLayout *vboxLayout1;
    QGroupBox *GroupBox10;
    QVBoxLayout *vboxLayout2;
    QHBoxLayout *hboxLayout;
    QLabel *TestLabel1;
    QLineEdit *_lineeditCommandFiles2;
    QGroupBox *_groupBoxGnuDiffOptions;
    QHBoxLayout *hboxLayout1;
    QVBoxLayout *vboxLayout3;
    QCheckBox *_checkboxIgnoreTrailingBlanks;
    QCheckBox *_checkboxIgnoreWhitespace;
    QCheckBox *_checkboxIgnoreCase;
    QCheckBox *_checkboxIgnoreBlankLines;
    QGroupBox *ButtonGroup8;
    QVBoxLayout *vboxLayout4;
    QRadioButton *_radiobuttonQualityNormal;
    QRadioButton *_radiobuttonQualityFastest;
    QRadioButton *_radiobuttonQualityHighest;
    QGroupBox *GroupBox11;
    QHBoxLayout *hboxLayout2;
    QHBoxLayout *hboxLayout3;
    QLabel *TestLabel1_3_2_2;
    QLineEdit *_lineeditCommandFiles3;
    QWidget *Widget4;
    QVBoxLayout *vboxLayout5;
    QHBoxLayout *hboxLayout4;
    QLabel *TestLabel1_2;
    QLineEdit *_lineeditCommandDirs;
    QHBoxLayout *hboxLayout5;
    QLabel *TestLabel1_2_2;
    QLineEdit *_lineeditCommandDirsRecursive;
    QGroupBox *GroupBox62;
    QCheckBox *_checkboxRecursive;
    QWidget *Widget5;
    QVBoxLayout *vboxLayout6;
    QHBoxLayout *hboxLayout6;
    QGroupBox *_groupboxFileDiffs;
    QVBoxLayout *vboxLayout7;
    QHBoxLayout *hboxLayout7;
    QLabel *TextLabel1;
    QComboBox *_comboHordiffType;
    QSpacerItem *Spacer54;
    QHBoxLayout *hboxLayout8;
    QLabel *TextLabel2;
    QSpinBox *_spinboxHordiffContext;
    QSpacerItem *Spacer37;
    QCheckBox *_checkboxIgnoreHorizontalWhitespace;
    QCheckBox *_checkboxHideCarriageReturns;
    QHBoxLayout *hboxLayout9;
    QLabel *_tabWidthlabel;
    QSpinBox *_spinboxTabWidth;
    QSpacerItem *Spacer1_2;
    QHBoxLayout *hboxLayout10;
    QCheckBox *_checkboxDrawVerticalLine;
    QSpacerItem *Spacer15;
    QLabel *_vlinePosLabel;
    QSpinBox *_spinboxVlinePos;
    QHBoxLayout *hboxLayout11;
    QLabel *TextLabel1_3;
    QComboBox *_comboIgnoreFile;
    QSpacerItem *Spacer55;
    QGroupBox *_groupboxDirectoryDiffs;
    QVBoxLayout *vboxLayout8;
    QHBoxLayout *hboxLayout12;
    QCheckBox *_checkboxIgnoreFileChanges;
    QSpacerItem *Spacer2;
    QGroupBox *GroupBox8_2_2;
    QVBoxLayout *vboxLayout9;
    QHBoxLayout *hboxLayout13;
    QCheckBox *_checkboxFormatClipboardText;
    QSpacerItem *Spacer7;
    QLabel *TextLabel1_2;
    QLineEdit *_lineeditClipboardFormat;
    QWidget *Widget6;
    QHBoxLayout *hboxLayout14;
    QListWidget *_listboxColors;
    QVBoxLayout *vboxLayout10;
    QGroupBox *GroupBox7;
    QHBoxLayout *hboxLayout15;
    QLabel *_labelEditBack;
    QPushButton *_buttonEditBack;
    QGroupBox *GroupBox7_2;
    QHBoxLayout *hboxLayout16;
    QLabel *_labelEditFore;
    QPushButton *_buttonEditFore;
    QGroupBox *GroupBox2;
    QHBoxLayout *hboxLayout17;
    QTextEdit *_labelDescription;
    QWidget *tab;
    QVBoxLayout *vboxLayout11;
    QGroupBox *GroupBox25_3;
    QVBoxLayout *vboxLayout12;
    QHBoxLayout *hboxLayout18;
    QLineEdit *_editFontTextName;
    QPushButton *_editFontText;
    QLabel *_labelFontText;
    QGroupBox *GroupBox25_2_2;
    QVBoxLayout *vboxLayout13;
    QHBoxLayout *hboxLayout19;
    QLineEdit *_editFontAppName;
    QPushButton *_editFontApp;
    QLabel *_labelFontApp;
    QHBoxLayout *hboxLayout20;
    QSpacerItem *Horizontal_Spacing2;
    QPushButton *_buttonApply;
    QPushButton *_buttonOk;
    QPushButton *_buttonCancel;

    void setupUi(QDialog *XxOptionsDialogBase)
    {
        if (XxOptionsDialogBase->objectName().isEmpty())
            XxOptionsDialogBase->setObjectName("XxOptionsDialogBase");
        XxOptionsDialogBase->resize(687, 450);
        XxOptionsDialogBase->setSizeGripEnabled(true);
        vboxLayout = new QVBoxLayout(XxOptionsDialogBase);
        vboxLayout->setSpacing(6);
        vboxLayout->setContentsMargins(11, 11, 11, 11);
        vboxLayout->setObjectName("vboxLayout");
        _tabWidget = new QTabWidget(XxOptionsDialogBase);
        _tabWidget->setObjectName("_tabWidget");
        Widget2 = new QWidget();
        Widget2->setObjectName("Widget2");
        vboxLayout1 = new QVBoxLayout(Widget2);
        vboxLayout1->setSpacing(6);
        vboxLayout1->setContentsMargins(11, 11, 11, 11);
        vboxLayout1->setObjectName("vboxLayout1");
        GroupBox10 = new QGroupBox(Widget2);
        GroupBox10->setObjectName("GroupBox10");
        vboxLayout2 = new QVBoxLayout(GroupBox10);
        vboxLayout2->setSpacing(6);
        vboxLayout2->setContentsMargins(11, 11, 11, 11);
        vboxLayout2->setObjectName("vboxLayout2");
        hboxLayout = new QHBoxLayout();
        hboxLayout->setSpacing(6);
        hboxLayout->setContentsMargins(0, 0, 0, 0);
        hboxLayout->setObjectName("hboxLayout");
        TestLabel1 = new QLabel(GroupBox10);
        TestLabel1->setObjectName("TestLabel1");
        TestLabel1->setWordWrap(false);

        hboxLayout->addWidget(TestLabel1);

        _lineeditCommandFiles2 = new QLineEdit(GroupBox10);
        _lineeditCommandFiles2->setObjectName("_lineeditCommandFiles2");

        hboxLayout->addWidget(_lineeditCommandFiles2);


        vboxLayout2->addLayout(hboxLayout);

        _groupBoxGnuDiffOptions = new QGroupBox(GroupBox10);
        _groupBoxGnuDiffOptions->setObjectName("_groupBoxGnuDiffOptions");
        hboxLayout1 = new QHBoxLayout(_groupBoxGnuDiffOptions);
        hboxLayout1->setSpacing(6);
        hboxLayout1->setContentsMargins(11, 11, 11, 11);
        hboxLayout1->setObjectName("hboxLayout1");
        vboxLayout3 = new QVBoxLayout();
        vboxLayout3->setSpacing(6);
        vboxLayout3->setContentsMargins(0, 0, 0, 0);
        vboxLayout3->setObjectName("vboxLayout3");
        _checkboxIgnoreTrailingBlanks = new QCheckBox(_groupBoxGnuDiffOptions);
        _checkboxIgnoreTrailingBlanks->setObjectName("_checkboxIgnoreTrailingBlanks");

        vboxLayout3->addWidget(_checkboxIgnoreTrailingBlanks);

        _checkboxIgnoreWhitespace = new QCheckBox(_groupBoxGnuDiffOptions);
        _checkboxIgnoreWhitespace->setObjectName("_checkboxIgnoreWhitespace");

        vboxLayout3->addWidget(_checkboxIgnoreWhitespace);

        _checkboxIgnoreCase = new QCheckBox(_groupBoxGnuDiffOptions);
        _checkboxIgnoreCase->setObjectName("_checkboxIgnoreCase");

        vboxLayout3->addWidget(_checkboxIgnoreCase);

        _checkboxIgnoreBlankLines = new QCheckBox(_groupBoxGnuDiffOptions);
        _checkboxIgnoreBlankLines->setObjectName("_checkboxIgnoreBlankLines");
        _checkboxIgnoreBlankLines->setEnabled(true);

        vboxLayout3->addWidget(_checkboxIgnoreBlankLines);


        hboxLayout1->addLayout(vboxLayout3);

        ButtonGroup8 = new QGroupBox(_groupBoxGnuDiffOptions);
        ButtonGroup8->setObjectName("ButtonGroup8");
        vboxLayout4 = new QVBoxLayout(ButtonGroup8);
        vboxLayout4->setSpacing(6);
        vboxLayout4->setContentsMargins(11, 11, 11, 11);
        vboxLayout4->setObjectName("vboxLayout4");
        _radiobuttonQualityNormal = new QRadioButton(ButtonGroup8);
        _radiobuttonQualityNormal->setObjectName("_radiobuttonQualityNormal");

        vboxLayout4->addWidget(_radiobuttonQualityNormal);

        _radiobuttonQualityFastest = new QRadioButton(ButtonGroup8);
        _radiobuttonQualityFastest->setObjectName("_radiobuttonQualityFastest");

        vboxLayout4->addWidget(_radiobuttonQualityFastest);

        _radiobuttonQualityHighest = new QRadioButton(ButtonGroup8);
        _radiobuttonQualityHighest->setObjectName("_radiobuttonQualityHighest");

        vboxLayout4->addWidget(_radiobuttonQualityHighest);


        hboxLayout1->addWidget(ButtonGroup8);


        vboxLayout2->addWidget(_groupBoxGnuDiffOptions);


        vboxLayout1->addWidget(GroupBox10);

        GroupBox11 = new QGroupBox(Widget2);
        GroupBox11->setObjectName("GroupBox11");
        hboxLayout2 = new QHBoxLayout(GroupBox11);
        hboxLayout2->setSpacing(6);
        hboxLayout2->setContentsMargins(11, 11, 11, 11);
        hboxLayout2->setObjectName("hboxLayout2");
        hboxLayout3 = new QHBoxLayout();
        hboxLayout3->setSpacing(6);
        hboxLayout3->setContentsMargins(0, 0, 0, 0);
        hboxLayout3->setObjectName("hboxLayout3");
        TestLabel1_3_2_2 = new QLabel(GroupBox11);
        TestLabel1_3_2_2->setObjectName("TestLabel1_3_2_2");
        TestLabel1_3_2_2->setWordWrap(false);

        hboxLayout3->addWidget(TestLabel1_3_2_2);

        _lineeditCommandFiles3 = new QLineEdit(GroupBox11);
        _lineeditCommandFiles3->setObjectName("_lineeditCommandFiles3");

        hboxLayout3->addWidget(_lineeditCommandFiles3);


        hboxLayout2->addLayout(hboxLayout3);


        vboxLayout1->addWidget(GroupBox11);

        _tabWidget->addTab(Widget2, QString());
        Widget4 = new QWidget();
        Widget4->setObjectName("Widget4");
        vboxLayout5 = new QVBoxLayout(Widget4);
        vboxLayout5->setSpacing(6);
        vboxLayout5->setContentsMargins(11, 11, 11, 11);
        vboxLayout5->setObjectName("vboxLayout5");
        hboxLayout4 = new QHBoxLayout();
        hboxLayout4->setSpacing(6);
        hboxLayout4->setContentsMargins(0, 0, 0, 0);
        hboxLayout4->setObjectName("hboxLayout4");
        TestLabel1_2 = new QLabel(Widget4);
        TestLabel1_2->setObjectName("TestLabel1_2");
        TestLabel1_2->setWordWrap(false);

        hboxLayout4->addWidget(TestLabel1_2);

        _lineeditCommandDirs = new QLineEdit(Widget4);
        _lineeditCommandDirs->setObjectName("_lineeditCommandDirs");

        hboxLayout4->addWidget(_lineeditCommandDirs);


        vboxLayout5->addLayout(hboxLayout4);

        hboxLayout5 = new QHBoxLayout();
        hboxLayout5->setSpacing(6);
        hboxLayout5->setContentsMargins(0, 0, 0, 0);
        hboxLayout5->setObjectName("hboxLayout5");
        TestLabel1_2_2 = new QLabel(Widget4);
        TestLabel1_2_2->setObjectName("TestLabel1_2_2");
        TestLabel1_2_2->setWordWrap(false);

        hboxLayout5->addWidget(TestLabel1_2_2);

        _lineeditCommandDirsRecursive = new QLineEdit(Widget4);
        _lineeditCommandDirsRecursive->setObjectName("_lineeditCommandDirsRecursive");

        hboxLayout5->addWidget(_lineeditCommandDirsRecursive);


        vboxLayout5->addLayout(hboxLayout5);

        GroupBox62 = new QGroupBox(Widget4);
        GroupBox62->setObjectName("GroupBox62");
        _checkboxRecursive = new QCheckBox(GroupBox62);
        _checkboxRecursive->setObjectName("_checkboxRecursive");
        _checkboxRecursive->setGeometry(QRect(20, 30, 104, 20));

        vboxLayout5->addWidget(GroupBox62);

        _tabWidget->addTab(Widget4, QString());
        Widget5 = new QWidget();
        Widget5->setObjectName("Widget5");
        vboxLayout6 = new QVBoxLayout(Widget5);
        vboxLayout6->setSpacing(6);
        vboxLayout6->setContentsMargins(11, 11, 11, 11);
        vboxLayout6->setObjectName("vboxLayout6");
        hboxLayout6 = new QHBoxLayout();
        hboxLayout6->setSpacing(6);
        hboxLayout6->setContentsMargins(0, 0, 0, 0);
        hboxLayout6->setObjectName("hboxLayout6");
        _groupboxFileDiffs = new QGroupBox(Widget5);
        _groupboxFileDiffs->setObjectName("_groupboxFileDiffs");
        vboxLayout7 = new QVBoxLayout(_groupboxFileDiffs);
        vboxLayout7->setSpacing(6);
        vboxLayout7->setContentsMargins(11, 11, 11, 11);
        vboxLayout7->setObjectName("vboxLayout7");
        hboxLayout7 = new QHBoxLayout();
        hboxLayout7->setSpacing(6);
        hboxLayout7->setContentsMargins(0, 0, 0, 0);
        hboxLayout7->setObjectName("hboxLayout7");
        TextLabel1 = new QLabel(_groupboxFileDiffs);
        TextLabel1->setObjectName("TextLabel1");
        TextLabel1->setWordWrap(false);

        hboxLayout7->addWidget(TextLabel1);

        _comboHordiffType = new QComboBox(_groupboxFileDiffs);
        _comboHordiffType->addItem(QString());
        _comboHordiffType->addItem(QString());
        _comboHordiffType->addItem(QString());
        _comboHordiffType->setObjectName("_comboHordiffType");

        hboxLayout7->addWidget(_comboHordiffType);

        Spacer54 = new QSpacerItem(20, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        hboxLayout7->addItem(Spacer54);


        vboxLayout7->addLayout(hboxLayout7);

        hboxLayout8 = new QHBoxLayout();
        hboxLayout8->setSpacing(6);
        hboxLayout8->setContentsMargins(0, 0, 0, 0);
        hboxLayout8->setObjectName("hboxLayout8");
        TextLabel2 = new QLabel(_groupboxFileDiffs);
        TextLabel2->setObjectName("TextLabel2");
        TextLabel2->setWordWrap(false);

        hboxLayout8->addWidget(TextLabel2);

        _spinboxHordiffContext = new QSpinBox(_groupboxFileDiffs);
        _spinboxHordiffContext->setObjectName("_spinboxHordiffContext");
        _spinboxHordiffContext->setMaximum(20);
        _spinboxHordiffContext->setMinimum(1);
        _spinboxHordiffContext->setValue(5);

        hboxLayout8->addWidget(_spinboxHordiffContext);

        Spacer37 = new QSpacerItem(20, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        hboxLayout8->addItem(Spacer37);


        vboxLayout7->addLayout(hboxLayout8);

        _checkboxIgnoreHorizontalWhitespace = new QCheckBox(_groupboxFileDiffs);
        _checkboxIgnoreHorizontalWhitespace->setObjectName("_checkboxIgnoreHorizontalWhitespace");

        vboxLayout7->addWidget(_checkboxIgnoreHorizontalWhitespace);

        _checkboxHideCarriageReturns = new QCheckBox(_groupboxFileDiffs);
        _checkboxHideCarriageReturns->setObjectName("_checkboxHideCarriageReturns");

        vboxLayout7->addWidget(_checkboxHideCarriageReturns);

        hboxLayout9 = new QHBoxLayout();
        hboxLayout9->setSpacing(6);
        hboxLayout9->setContentsMargins(0, 0, 0, 0);
        hboxLayout9->setObjectName("hboxLayout9");
        _tabWidthlabel = new QLabel(_groupboxFileDiffs);
        _tabWidthlabel->setObjectName("_tabWidthlabel");
        _tabWidthlabel->setWordWrap(false);

        hboxLayout9->addWidget(_tabWidthlabel);

        _spinboxTabWidth = new QSpinBox(_groupboxFileDiffs);
        _spinboxTabWidth->setObjectName("_spinboxTabWidth");
        _spinboxTabWidth->setMaximum(48);
        _spinboxTabWidth->setMinimum(0);
        _spinboxTabWidth->setValue(8);

        hboxLayout9->addWidget(_spinboxTabWidth);

        Spacer1_2 = new QSpacerItem(20, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        hboxLayout9->addItem(Spacer1_2);


        vboxLayout7->addLayout(hboxLayout9);

        hboxLayout10 = new QHBoxLayout();
        hboxLayout10->setSpacing(6);
        hboxLayout10->setContentsMargins(0, 0, 0, 0);
        hboxLayout10->setObjectName("hboxLayout10");
        _checkboxDrawVerticalLine = new QCheckBox(_groupboxFileDiffs);
        _checkboxDrawVerticalLine->setObjectName("_checkboxDrawVerticalLine");

        hboxLayout10->addWidget(_checkboxDrawVerticalLine);

        Spacer15 = new QSpacerItem(20, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        hboxLayout10->addItem(Spacer15);

        _vlinePosLabel = new QLabel(_groupboxFileDiffs);
        _vlinePosLabel->setObjectName("_vlinePosLabel");
        _vlinePosLabel->setWordWrap(false);

        hboxLayout10->addWidget(_vlinePosLabel);

        _spinboxVlinePos = new QSpinBox(_groupboxFileDiffs);
        _spinboxVlinePos->setObjectName("_spinboxVlinePos");
        _spinboxVlinePos->setMaximum(512);
        _spinboxVlinePos->setMinimum(1);
        _spinboxVlinePos->setValue(80);

        hboxLayout10->addWidget(_spinboxVlinePos);


        vboxLayout7->addLayout(hboxLayout10);

        hboxLayout11 = new QHBoxLayout();
        hboxLayout11->setSpacing(6);
        hboxLayout11->setContentsMargins(0, 0, 0, 0);
        hboxLayout11->setObjectName("hboxLayout11");
        TextLabel1_3 = new QLabel(_groupboxFileDiffs);
        TextLabel1_3->setObjectName("TextLabel1_3");
        TextLabel1_3->setWordWrap(false);

        hboxLayout11->addWidget(TextLabel1_3);

        _comboIgnoreFile = new QComboBox(_groupboxFileDiffs);
        _comboIgnoreFile->addItem(QString());
        _comboIgnoreFile->addItem(QString());
        _comboIgnoreFile->addItem(QString());
        _comboIgnoreFile->addItem(QString());
        _comboIgnoreFile->setObjectName("_comboIgnoreFile");

        hboxLayout11->addWidget(_comboIgnoreFile);

        Spacer55 = new QSpacerItem(20, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        hboxLayout11->addItem(Spacer55);


        vboxLayout7->addLayout(hboxLayout11);


        hboxLayout6->addWidget(_groupboxFileDiffs);

        _groupboxDirectoryDiffs = new QGroupBox(Widget5);
        _groupboxDirectoryDiffs->setObjectName("_groupboxDirectoryDiffs");
        vboxLayout8 = new QVBoxLayout(_groupboxDirectoryDiffs);
        vboxLayout8->setSpacing(6);
        vboxLayout8->setContentsMargins(11, 11, 11, 11);
        vboxLayout8->setObjectName("vboxLayout8");
        hboxLayout12 = new QHBoxLayout();
        hboxLayout12->setSpacing(6);
        hboxLayout12->setContentsMargins(0, 0, 0, 0);
        hboxLayout12->setObjectName("hboxLayout12");
        _checkboxIgnoreFileChanges = new QCheckBox(_groupboxDirectoryDiffs);
        _checkboxIgnoreFileChanges->setObjectName("_checkboxIgnoreFileChanges");

        hboxLayout12->addWidget(_checkboxIgnoreFileChanges);

        Spacer2 = new QSpacerItem(20, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        hboxLayout12->addItem(Spacer2);


        vboxLayout8->addLayout(hboxLayout12);


        hboxLayout6->addWidget(_groupboxDirectoryDiffs);


        vboxLayout6->addLayout(hboxLayout6);

        GroupBox8_2_2 = new QGroupBox(Widget5);
        GroupBox8_2_2->setObjectName("GroupBox8_2_2");
        vboxLayout9 = new QVBoxLayout(GroupBox8_2_2);
        vboxLayout9->setSpacing(6);
        vboxLayout9->setContentsMargins(11, 11, 11, 11);
        vboxLayout9->setObjectName("vboxLayout9");
        hboxLayout13 = new QHBoxLayout();
        hboxLayout13->setSpacing(4);
        hboxLayout13->setContentsMargins(0, 0, 0, 0);
        hboxLayout13->setObjectName("hboxLayout13");
        _checkboxFormatClipboardText = new QCheckBox(GroupBox8_2_2);
        _checkboxFormatClipboardText->setObjectName("_checkboxFormatClipboardText");

        hboxLayout13->addWidget(_checkboxFormatClipboardText);

        Spacer7 = new QSpacerItem(20, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        hboxLayout13->addItem(Spacer7);

        TextLabel1_2 = new QLabel(GroupBox8_2_2);
        TextLabel1_2->setObjectName("TextLabel1_2");
        TextLabel1_2->setWordWrap(false);

        hboxLayout13->addWidget(TextLabel1_2);

        _lineeditClipboardFormat = new QLineEdit(GroupBox8_2_2);
        _lineeditClipboardFormat->setObjectName("_lineeditClipboardFormat");

        hboxLayout13->addWidget(_lineeditClipboardFormat);


        vboxLayout9->addLayout(hboxLayout13);


        vboxLayout6->addWidget(GroupBox8_2_2);

        _tabWidget->addTab(Widget5, QString());
        Widget6 = new QWidget();
        Widget6->setObjectName("Widget6");
        hboxLayout14 = new QHBoxLayout(Widget6);
        hboxLayout14->setSpacing(6);
        hboxLayout14->setContentsMargins(11, 11, 11, 11);
        hboxLayout14->setObjectName("hboxLayout14");
        _listboxColors = new QListWidget(Widget6);
        _listboxColors->setObjectName("_listboxColors");

        hboxLayout14->addWidget(_listboxColors);

        vboxLayout10 = new QVBoxLayout();
        vboxLayout10->setSpacing(6);
        vboxLayout10->setContentsMargins(0, 0, 0, 0);
        vboxLayout10->setObjectName("vboxLayout10");
        GroupBox7 = new QGroupBox(Widget6);
        GroupBox7->setObjectName("GroupBox7");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(GroupBox7->sizePolicy().hasHeightForWidth());
        GroupBox7->setSizePolicy(sizePolicy);
        hboxLayout15 = new QHBoxLayout(GroupBox7);
        hboxLayout15->setSpacing(6);
        hboxLayout15->setContentsMargins(11, 11, 11, 11);
        hboxLayout15->setObjectName("hboxLayout15");
        _labelEditBack = new QLabel(GroupBox7);
        _labelEditBack->setObjectName("_labelEditBack");
        _labelEditBack->setMinimumSize(QSize(100, 0));
        _labelEditBack->setFrameShape(QFrame::Box);
        _labelEditBack->setFrameShadow(QFrame::Sunken);
        _labelEditBack->setLineWidth(2);
        _labelEditBack->setWordWrap(false);

        hboxLayout15->addWidget(_labelEditBack);

        _buttonEditBack = new QPushButton(GroupBox7);
        _buttonEditBack->setObjectName("_buttonEditBack");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Preferred);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(_buttonEditBack->sizePolicy().hasHeightForWidth());
        _buttonEditBack->setSizePolicy(sizePolicy1);

        hboxLayout15->addWidget(_buttonEditBack);


        vboxLayout10->addWidget(GroupBox7);

        GroupBox7_2 = new QGroupBox(Widget6);
        GroupBox7_2->setObjectName("GroupBox7_2");
        sizePolicy.setHeightForWidth(GroupBox7_2->sizePolicy().hasHeightForWidth());
        GroupBox7_2->setSizePolicy(sizePolicy);
        GroupBox7_2->setMinimumSize(QSize(0, 0));
        hboxLayout16 = new QHBoxLayout(GroupBox7_2);
        hboxLayout16->setSpacing(6);
        hboxLayout16->setContentsMargins(11, 11, 11, 11);
        hboxLayout16->setObjectName("hboxLayout16");
        _labelEditFore = new QLabel(GroupBox7_2);
        _labelEditFore->setObjectName("_labelEditFore");
        _labelEditFore->setMinimumSize(QSize(100, 0));
        _labelEditFore->setFrameShape(QFrame::Box);
        _labelEditFore->setFrameShadow(QFrame::Sunken);
        _labelEditFore->setLineWidth(2);
        _labelEditFore->setWordWrap(false);

        hboxLayout16->addWidget(_labelEditFore);

        _buttonEditFore = new QPushButton(GroupBox7_2);
        _buttonEditFore->setObjectName("_buttonEditFore");
        sizePolicy1.setHeightForWidth(_buttonEditFore->sizePolicy().hasHeightForWidth());
        _buttonEditFore->setSizePolicy(sizePolicy1);

        hboxLayout16->addWidget(_buttonEditFore);


        vboxLayout10->addWidget(GroupBox7_2);

        GroupBox2 = new QGroupBox(Widget6);
        GroupBox2->setObjectName("GroupBox2");
        QSizePolicy sizePolicy2(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Preferred);
        sizePolicy2.setHorizontalStretch(0);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(GroupBox2->sizePolicy().hasHeightForWidth());
        GroupBox2->setSizePolicy(sizePolicy2);
        GroupBox2->setMinimumSize(QSize(0, 100));
        hboxLayout17 = new QHBoxLayout(GroupBox2);
        hboxLayout17->setSpacing(6);
        hboxLayout17->setContentsMargins(11, 11, 11, 11);
        hboxLayout17->setObjectName("hboxLayout17");
        _labelDescription = new QTextEdit(GroupBox2);
        _labelDescription->setObjectName("_labelDescription");
        _labelDescription->setFrameShape(QFrame::Box);
        _labelDescription->setFrameShadow(QFrame::Plain);
        _labelDescription->setAcceptRichText(true);

        hboxLayout17->addWidget(_labelDescription);


        vboxLayout10->addWidget(GroupBox2);


        hboxLayout14->addLayout(vboxLayout10);

        _tabWidget->addTab(Widget6, QString());
        tab = new QWidget();
        tab->setObjectName("tab");
        vboxLayout11 = new QVBoxLayout(tab);
        vboxLayout11->setSpacing(6);
        vboxLayout11->setContentsMargins(11, 11, 11, 11);
        vboxLayout11->setObjectName("vboxLayout11");
        GroupBox25_3 = new QGroupBox(tab);
        GroupBox25_3->setObjectName("GroupBox25_3");
        vboxLayout12 = new QVBoxLayout(GroupBox25_3);
        vboxLayout12->setSpacing(6);
        vboxLayout12->setContentsMargins(11, 11, 11, 11);
        vboxLayout12->setObjectName("vboxLayout12");
        hboxLayout18 = new QHBoxLayout();
        hboxLayout18->setSpacing(6);
        hboxLayout18->setObjectName("hboxLayout18");
        _editFontTextName = new QLineEdit(GroupBox25_3);
        _editFontTextName->setObjectName("_editFontTextName");
        QSizePolicy sizePolicy3(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Fixed);
        sizePolicy3.setHorizontalStretch(10);
        sizePolicy3.setVerticalStretch(0);
        sizePolicy3.setHeightForWidth(_editFontTextName->sizePolicy().hasHeightForWidth());
        _editFontTextName->setSizePolicy(sizePolicy3);
        _editFontTextName->setDragEnabled(false);
        _editFontTextName->setReadOnly(true);

        hboxLayout18->addWidget(_editFontTextName);

        _editFontText = new QPushButton(GroupBox25_3);
        _editFontText->setObjectName("_editFontText");
        QSizePolicy sizePolicy4(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Fixed);
        sizePolicy4.setHorizontalStretch(0);
        sizePolicy4.setVerticalStretch(0);
        sizePolicy4.setHeightForWidth(_editFontText->sizePolicy().hasHeightForWidth());
        _editFontText->setSizePolicy(sizePolicy4);

        hboxLayout18->addWidget(_editFontText);


        vboxLayout12->addLayout(hboxLayout18);

        _labelFontText = new QLabel(GroupBox25_3);
        _labelFontText->setObjectName("_labelFontText");
        _labelFontText->setFrameShape(QFrame::Box);
        _labelFontText->setFrameShadow(QFrame::Plain);
        _labelFontText->setTextFormat(Qt::PlainText);
        _labelFontText->setAlignment(Qt::AlignCenter);
        _labelFontText->setWordWrap(false);

        vboxLayout12->addWidget(_labelFontText);


        vboxLayout11->addWidget(GroupBox25_3);

        GroupBox25_2_2 = new QGroupBox(tab);
        GroupBox25_2_2->setObjectName("GroupBox25_2_2");
        vboxLayout13 = new QVBoxLayout(GroupBox25_2_2);
        vboxLayout13->setSpacing(6);
        vboxLayout13->setContentsMargins(11, 11, 11, 11);
        vboxLayout13->setObjectName("vboxLayout13");
        hboxLayout19 = new QHBoxLayout();
        hboxLayout19->setSpacing(6);
        hboxLayout19->setObjectName("hboxLayout19");
        _editFontAppName = new QLineEdit(GroupBox25_2_2);
        _editFontAppName->setObjectName("_editFontAppName");
        _editFontAppName->setReadOnly(true);

        hboxLayout19->addWidget(_editFontAppName);

        _editFontApp = new QPushButton(GroupBox25_2_2);
        _editFontApp->setObjectName("_editFontApp");
        sizePolicy4.setHeightForWidth(_editFontApp->sizePolicy().hasHeightForWidth());
        _editFontApp->setSizePolicy(sizePolicy4);

        hboxLayout19->addWidget(_editFontApp);


        vboxLayout13->addLayout(hboxLayout19);

        _labelFontApp = new QLabel(GroupBox25_2_2);
        _labelFontApp->setObjectName("_labelFontApp");
        _labelFontApp->setFrameShape(QFrame::Box);
        _labelFontApp->setFrameShadow(QFrame::Plain);
        _labelFontApp->setTextFormat(Qt::PlainText);
        _labelFontApp->setAlignment(Qt::AlignCenter);
        _labelFontApp->setWordWrap(false);

        vboxLayout13->addWidget(_labelFontApp);


        vboxLayout11->addWidget(GroupBox25_2_2);

        _tabWidget->addTab(tab, QString());

        vboxLayout->addWidget(_tabWidget);

        hboxLayout20 = new QHBoxLayout();
        hboxLayout20->setSpacing(6);
        hboxLayout20->setContentsMargins(0, 0, 0, 0);
        hboxLayout20->setObjectName("hboxLayout20");
        Horizontal_Spacing2 = new QSpacerItem(20, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        hboxLayout20->addItem(Horizontal_Spacing2);

        _buttonApply = new QPushButton(XxOptionsDialogBase);
        _buttonApply->setObjectName("_buttonApply");
        _buttonApply->setAutoDefault(true);

        hboxLayout20->addWidget(_buttonApply);

        _buttonOk = new QPushButton(XxOptionsDialogBase);
        _buttonOk->setObjectName("_buttonOk");
        _buttonOk->setAutoDefault(true);

        hboxLayout20->addWidget(_buttonOk);

        _buttonCancel = new QPushButton(XxOptionsDialogBase);
        _buttonCancel->setObjectName("_buttonCancel");
        _buttonCancel->setAutoDefault(true);

        hboxLayout20->addWidget(_buttonCancel);


        vboxLayout->addLayout(hboxLayout20);


        retranslateUi(XxOptionsDialogBase);
        QObject::connect(_buttonOk, &QPushButton::clicked, XxOptionsDialogBase, qOverload<>(&QDialog::accept));
        QObject::connect(_buttonCancel, &QPushButton::clicked, XxOptionsDialogBase, qOverload<>(&QDialog::reject));

        _buttonOk->setDefault(true);


        QMetaObject::connectSlotsByName(XxOptionsDialogBase);
    } // setupUi

    void retranslateUi(QDialog *XxOptionsDialogBase)
    {
        XxOptionsDialogBase->setWindowTitle(QCoreApplication::translate("XxOptionsDialogBase", "Options", nullptr));
        GroupBox10->setTitle(QCoreApplication::translate("XxOptionsDialogBase", "2-way diff", nullptr));
        TestLabel1->setText(QCoreApplication::translate("XxOptionsDialogBase", "Command:", nullptr));
        _groupBoxGnuDiffOptions->setTitle(QCoreApplication::translate("XxOptionsDialogBase", "GNU diff options", nullptr));
        _checkboxIgnoreTrailingBlanks->setText(QCoreApplication::translate("XxOptionsDialogBase", "Ignore trailing blanks", nullptr));
        _checkboxIgnoreWhitespace->setText(QCoreApplication::translate("XxOptionsDialogBase", "Ignore whitespace", nullptr));
        _checkboxIgnoreCase->setText(QCoreApplication::translate("XxOptionsDialogBase", "Ignore case", nullptr));
        _checkboxIgnoreBlankLines->setText(QCoreApplication::translate("XxOptionsDialogBase", "Ignore blank lines", nullptr));
        ButtonGroup8->setTitle(QCoreApplication::translate("XxOptionsDialogBase", "Quality", nullptr));
        _radiobuttonQualityNormal->setText(QCoreApplication::translate("XxOptionsDialogBase", "Normal", nullptr));
        _radiobuttonQualityFastest->setText(QCoreApplication::translate("XxOptionsDialogBase", "Fastest", nullptr));
        _radiobuttonQualityHighest->setText(QCoreApplication::translate("XxOptionsDialogBase", "Highest", nullptr));
        GroupBox11->setTitle(QCoreApplication::translate("XxOptionsDialogBase", "3-way diff", nullptr));
        TestLabel1_3_2_2->setText(QCoreApplication::translate("XxOptionsDialogBase", "Command:", nullptr));
        _tabWidget->setTabText(_tabWidget->indexOf(Widget2), QCoreApplication::translate("XxOptionsDialogBase", "File diff", nullptr));
        TestLabel1_2->setText(QCoreApplication::translate("XxOptionsDialogBase", "Shallow command:", nullptr));
        _lineeditCommandDirs->setText(QString());
        TestLabel1_2_2->setText(QCoreApplication::translate("XxOptionsDialogBase", "Recursive command:", nullptr));
        GroupBox62->setTitle(QCoreApplication::translate("XxOptionsDialogBase", "Options", nullptr));
        _checkboxRecursive->setText(QCoreApplication::translate("XxOptionsDialogBase", "Recursive", nullptr));
        _tabWidget->setTabText(_tabWidget->indexOf(Widget4), QCoreApplication::translate("XxOptionsDialogBase", "Directory diff", nullptr));
        _groupboxFileDiffs->setTitle(QCoreApplication::translate("XxOptionsDialogBase", "File diffs", nullptr));
        TextLabel1->setText(QCoreApplication::translate("XxOptionsDialogBase", "Horizontal diff:", nullptr));
        _comboHordiffType->setItemText(0, QCoreApplication::translate("XxOptionsDialogBase", "None", nullptr));
        _comboHordiffType->setItemText(1, QCoreApplication::translate("XxOptionsDialogBase", "Single", nullptr));
        _comboHordiffType->setItemText(2, QCoreApplication::translate("XxOptionsDialogBase", "Multiple", nullptr));

        TextLabel2->setText(QCoreApplication::translate("XxOptionsDialogBase", "Hor.diff. min.context:", nullptr));
        _checkboxIgnoreHorizontalWhitespace->setText(QCoreApplication::translate("XxOptionsDialogBase", "Ignore horizontal whitespace", nullptr));
        _checkboxHideCarriageReturns->setText(QCoreApplication::translate("XxOptionsDialogBase", "Hide carriage returns", nullptr));
        _tabWidthlabel->setText(QCoreApplication::translate("XxOptionsDialogBase", "Tab width:", nullptr));
        _checkboxDrawVerticalLine->setText(QCoreApplication::translate("XxOptionsDialogBase", "Draw vertical line", nullptr));
        _vlinePosLabel->setText(QCoreApplication::translate("XxOptionsDialogBase", "Position:", nullptr));
        TextLabel1_3->setText(QCoreApplication::translate("XxOptionsDialogBase", "Ignore file (3-way only):", nullptr));
        _comboIgnoreFile->setItemText(0, QCoreApplication::translate("XxOptionsDialogBase", "Ignore none", nullptr));
        _comboIgnoreFile->setItemText(1, QCoreApplication::translate("XxOptionsDialogBase", "Ignore left file", nullptr));
        _comboIgnoreFile->setItemText(2, QCoreApplication::translate("XxOptionsDialogBase", "Ignore middle file", nullptr));
        _comboIgnoreFile->setItemText(3, QCoreApplication::translate("XxOptionsDialogBase", "Ignore right file", nullptr));

        _groupboxDirectoryDiffs->setTitle(QCoreApplication::translate("XxOptionsDialogBase", "Directory diffs", nullptr));
        _checkboxIgnoreFileChanges->setText(QCoreApplication::translate("XxOptionsDialogBase", "Ignore file changes", nullptr));
        GroupBox8_2_2->setTitle(QCoreApplication::translate("XxOptionsDialogBase", "Common", nullptr));
        _checkboxFormatClipboardText->setText(QCoreApplication::translate("XxOptionsDialogBase", "Format clipboard text", nullptr));
        TextLabel1_2->setText(QCoreApplication::translate("XxOptionsDialogBase", "Clipboard format:", nullptr));
        _tabWidget->setTabText(_tabWidget->indexOf(Widget5), QCoreApplication::translate("XxOptionsDialogBase", "Display options", nullptr));
        GroupBox7->setTitle(QCoreApplication::translate("XxOptionsDialogBase", "Background", nullptr));
        _labelEditBack->setText(QString());
        _buttonEditBack->setText(QCoreApplication::translate("XxOptionsDialogBase", "Edit", nullptr));
        GroupBox7_2->setTitle(QCoreApplication::translate("XxOptionsDialogBase", "Foreground", nullptr));
        _labelEditFore->setText(QString());
        _buttonEditFore->setText(QCoreApplication::translate("XxOptionsDialogBase", "Edit", nullptr));
        GroupBox2->setTitle(QCoreApplication::translate("XxOptionsDialogBase", "Description", nullptr));
        _tabWidget->setTabText(_tabWidget->indexOf(Widget6), QCoreApplication::translate("XxOptionsDialogBase", "Colors", nullptr));
        GroupBox25_3->setTitle(QCoreApplication::translate("XxOptionsDialogBase", "Text Font", nullptr));
        _editFontText->setText(QCoreApplication::translate("XxOptionsDialogBase", "Edit", nullptr));
        _labelFontText->setText(QCoreApplication::translate("XxOptionsDialogBase", "Playing jazz vibe chords quickly excites my wife.", nullptr));
        GroupBox25_2_2->setTitle(QCoreApplication::translate("XxOptionsDialogBase", "App Font", nullptr));
        _editFontApp->setText(QCoreApplication::translate("XxOptionsDialogBase", "Edit", nullptr));
        _labelFontApp->setText(QCoreApplication::translate("XxOptionsDialogBase", "Playing jazz vibe chords quickly excites my wife.", nullptr));
        _tabWidget->setTabText(_tabWidget->indexOf(tab), QCoreApplication::translate("XxOptionsDialogBase", "Fonts", nullptr));
        _buttonApply->setText(QCoreApplication::translate("XxOptionsDialogBase", "Apply (redo diff)", nullptr));
        _buttonOk->setText(QCoreApplication::translate("XxOptionsDialogBase", "&OK", nullptr));
        _buttonCancel->setText(QCoreApplication::translate("XxOptionsDialogBase", "&Cancel", nullptr));
    } // retranslateUi

};

namespace Ui {
    class XxOptionsDialogBase: public Ui_XxOptionsDialogBase {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_OPTIONSDIALOGBASE_H
