/****************************************************************************
** Meta object code from reading C++ file 'app.h'
**
** Created by: The Qt Meta Object Compiler version 68 (Qt 6.7.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "app.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'app.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 68
#error "This file was generated using the moc from 6.7.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {

#ifdef QT_MOC_HAS_STRINGDATA
struct qt_meta_stringdata_CLASSXxAppENDCLASS_t {};
constexpr auto qt_meta_stringdata_CLASSXxAppENDCLASS = QtMocHelpers::stringData(
    "XxApp",
    "cursorChanged",
    "",
    "cursorLine",
    "textSizeChanged",
    "updateWidgets",
    "updateWidgetsForDiffs",
    "onNbLinesChanged",
    "openLeft",
    "openMiddle",
    "openRight",
    "saveAsLeft",
    "saveAsMiddle",
    "saveAsRight",
    "saveAsMerged",
    "saveAs",
    "generatePatchFromLeft",
    "generatePatchFromMiddle",
    "generatePatchFromRight",
    "saveSelectedOnly",
    "editLeft",
    "editMiddle",
    "editRight",
    "saveOptions",
    "quit",
    "search",
    "searchForward",
    "searchBackward",
    "redoDiff",
    "editDiffOptions",
    "editDisplayOptions",
    "diffFilesAtCursor",
    "nextAndDiffFiles",
    "copyFileLeftToRight",
    "copyFileRightToLeft",
    "removeFileLeft",
    "removeFileRight",
    "nextDifference",
    "previousDifference",
    "nextUnselected",
    "previousUnselected",
    "selectGlobalLeft",
    "selectGlobalMiddle",
    "selectGlobalRight",
    "selectGlobalNeither",
    "selectGlobalUnselect",
    "selectGlobalUnselectedLeft",
    "selectGlobalUnselectedMiddle",
    "selectGlobalUnselectedRight",
    "selectGlobalUnselectedNeither",
    "selectGlobalMerge",
    "selectRegionLeft",
    "selectRegionMiddle",
    "selectRegionRight",
    "selectRegionNeither",
    "selectRegionUnselect",
    "selectRegionLeftAndNext",
    "selectRegionMiddleAndNext",
    "selectRegionRightAndNext",
    "selectRegionNeitherAndNext",
    "regionSplitSwapJoin",
    "selectLineLeft",
    "selectLineMiddle",
    "selectLineRight",
    "selectLineNeither",
    "selectLineUnselect",
    "tabsAt3",
    "tabsAt4",
    "tabsAt8",
    "ignoreTrailing",
    "ignoreWhitespace",
    "ignoreCase",
    "ignoreBlankLines",
    "hideCarriageReturns",
    "dirDiffRecursive",
    "ignoreFileChanges",
    "qualityNormal",
    "qualityFastest",
    "qualityHighest",
    "toggleShowFilenames",
    "togglePaneMergedView",
    "togglePopupMergedView",
    "toggleToolbar",
    "toggleLineNumbers",
    "toggleVerticalLine",
    "toggleHorizNullMarkers",
    "toggleOverview",
    "hordiffTypeNone",
    "hordiffTypeSingle",
    "hordiffTypeMultiple",
    "hordiffTypeUpdate",
    "toggleIgnoreHorizontalWs",
    "toggleIgnorePerHunkWs",
    "toggleFormatClipboardText",
    "ignoreFileNone",
    "ignoreFileLeft",
    "ignoreFileMiddle",
    "ignoreFileRight",
    "helpManPage",
    "helpAbout",
    "pageDown",
    "pageUp",
    "cursorDown",
    "cursorUp",
    "cursorTop",
    "cursorBottom",
    "quitAccept",
    "quitReject",
    "quitMerged",
    "fontSizeIncrease",
    "fontSizeDecrease"
);
#else  // !QT_MOC_HAS_STRINGDATA
#error "qtmochelpers.h not found or too old."
#endif // !QT_MOC_HAS_STRINGDATA
} // unnamed namespace

Q_CONSTINIT static const uint qt_meta_data_CLASSXxAppENDCLASS[] = {

 // content:
      12,       // revision
       0,       // classname
       0,    0, // classinfo
     108,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       2,       // signalCount

 // signals: name, argc, parameters, tag, flags, initial metatype offsets
       1,    1,  662,    2, 0x06,    1 /* Public */,
       4,    0,  665,    2, 0x06,    3 /* Public */,

 // slots: name, argc, parameters, tag, flags, initial metatype offsets
       5,    0,  666,    2, 0x0a,    4 /* Public */,
       6,    0,  667,    2, 0x0a,    5 /* Public */,
       7,    0,  668,    2, 0x0a,    6 /* Public */,
       8,    0,  669,    2, 0x0a,    7 /* Public */,
       9,    0,  670,    2, 0x0a,    8 /* Public */,
      10,    0,  671,    2, 0x0a,    9 /* Public */,
      11,    0,  672,    2, 0x0a,   10 /* Public */,
      12,    0,  673,    2, 0x0a,   11 /* Public */,
      13,    0,  674,    2, 0x0a,   12 /* Public */,
      14,    0,  675,    2, 0x0a,   13 /* Public */,
      15,    0,  676,    2, 0x0a,   14 /* Public */,
      16,    0,  677,    2, 0x0a,   15 /* Public */,
      17,    0,  678,    2, 0x0a,   16 /* Public */,
      18,    0,  679,    2, 0x0a,   17 /* Public */,
      19,    0,  680,    2, 0x0a,   18 /* Public */,
      20,    0,  681,    2, 0x0a,   19 /* Public */,
      21,    0,  682,    2, 0x0a,   20 /* Public */,
      22,    0,  683,    2, 0x0a,   21 /* Public */,
      23,    0,  684,    2, 0x0a,   22 /* Public */,
      24,    0,  685,    2, 0x0a,   23 /* Public */,
      25,    0,  686,    2, 0x0a,   24 /* Public */,
      26,    0,  687,    2, 0x0a,   25 /* Public */,
      27,    0,  688,    2, 0x0a,   26 /* Public */,
      28,    0,  689,    2, 0x0a,   27 /* Public */,
      29,    0,  690,    2, 0x0a,   28 /* Public */,
      30,    0,  691,    2, 0x0a,   29 /* Public */,
      31,    0,  692,    2, 0x0a,   30 /* Public */,
      32,    0,  693,    2, 0x0a,   31 /* Public */,
      33,    0,  694,    2, 0x0a,   32 /* Public */,
      34,    0,  695,    2, 0x0a,   33 /* Public */,
      35,    0,  696,    2, 0x0a,   34 /* Public */,
      36,    0,  697,    2, 0x0a,   35 /* Public */,
      37,    0,  698,    2, 0x0a,   36 /* Public */,
      38,    0,  699,    2, 0x0a,   37 /* Public */,
      39,    0,  700,    2, 0x0a,   38 /* Public */,
      40,    0,  701,    2, 0x0a,   39 /* Public */,
      41,    0,  702,    2, 0x0a,   40 /* Public */,
      42,    0,  703,    2, 0x0a,   41 /* Public */,
      43,    0,  704,    2, 0x0a,   42 /* Public */,
      44,    0,  705,    2, 0x0a,   43 /* Public */,
      45,    0,  706,    2, 0x0a,   44 /* Public */,
      46,    0,  707,    2, 0x0a,   45 /* Public */,
      47,    0,  708,    2, 0x0a,   46 /* Public */,
      48,    0,  709,    2, 0x0a,   47 /* Public */,
      49,    0,  710,    2, 0x0a,   48 /* Public */,
      50,    0,  711,    2, 0x0a,   49 /* Public */,
      51,    0,  712,    2, 0x0a,   50 /* Public */,
      52,    0,  713,    2, 0x0a,   51 /* Public */,
      53,    0,  714,    2, 0x0a,   52 /* Public */,
      54,    0,  715,    2, 0x0a,   53 /* Public */,
      55,    0,  716,    2, 0x0a,   54 /* Public */,
      56,    0,  717,    2, 0x0a,   55 /* Public */,
      57,    0,  718,    2, 0x0a,   56 /* Public */,
      58,    0,  719,    2, 0x0a,   57 /* Public */,
      59,    0,  720,    2, 0x0a,   58 /* Public */,
      60,    0,  721,    2, 0x0a,   59 /* Public */,
      61,    0,  722,    2, 0x0a,   60 /* Public */,
      62,    0,  723,    2, 0x0a,   61 /* Public */,
      63,    0,  724,    2, 0x0a,   62 /* Public */,
      64,    0,  725,    2, 0x0a,   63 /* Public */,
      65,    0,  726,    2, 0x0a,   64 /* Public */,
      66,    0,  727,    2, 0x0a,   65 /* Public */,
      67,    0,  728,    2, 0x0a,   66 /* Public */,
      68,    0,  729,    2, 0x0a,   67 /* Public */,
      69,    0,  730,    2, 0x0a,   68 /* Public */,
      70,    0,  731,    2, 0x0a,   69 /* Public */,
      71,    0,  732,    2, 0x0a,   70 /* Public */,
      72,    0,  733,    2, 0x0a,   71 /* Public */,
      73,    0,  734,    2, 0x0a,   72 /* Public */,
      74,    0,  735,    2, 0x0a,   73 /* Public */,
      75,    0,  736,    2, 0x0a,   74 /* Public */,
      76,    0,  737,    2, 0x0a,   75 /* Public */,
      77,    0,  738,    2, 0x0a,   76 /* Public */,
      78,    0,  739,    2, 0x0a,   77 /* Public */,
      79,    0,  740,    2, 0x0a,   78 /* Public */,
      80,    0,  741,    2, 0x0a,   79 /* Public */,
      81,    0,  742,    2, 0x0a,   80 /* Public */,
      82,    0,  743,    2, 0x0a,   81 /* Public */,
      83,    0,  744,    2, 0x0a,   82 /* Public */,
      84,    0,  745,    2, 0x0a,   83 /* Public */,
      85,    0,  746,    2, 0x0a,   84 /* Public */,
      86,    0,  747,    2, 0x0a,   85 /* Public */,
      87,    0,  748,    2, 0x0a,   86 /* Public */,
      88,    0,  749,    2, 0x0a,   87 /* Public */,
      89,    0,  750,    2, 0x0a,   88 /* Public */,
      90,    0,  751,    2, 0x0a,   89 /* Public */,
      91,    0,  752,    2, 0x0a,   90 /* Public */,
      92,    0,  753,    2, 0x0a,   91 /* Public */,
      93,    0,  754,    2, 0x0a,   92 /* Public */,
      94,    0,  755,    2, 0x0a,   93 /* Public */,
      95,    0,  756,    2, 0x0a,   94 /* Public */,
      96,    0,  757,    2, 0x0a,   95 /* Public */,
      97,    0,  758,    2, 0x0a,   96 /* Public */,
      98,    0,  759,    2, 0x0a,   97 /* Public */,
      99,    0,  760,    2, 0x0a,   98 /* Public */,
     100,    0,  761,    2, 0x0a,   99 /* Public */,
     101,    0,  762,    2, 0x0a,  100 /* Public */,
     102,    0,  763,    2, 0x0a,  101 /* Public */,
     103,    0,  764,    2, 0x0a,  102 /* Public */,
     104,    0,  765,    2, 0x0a,  103 /* Public */,
     105,    0,  766,    2, 0x0a,  104 /* Public */,
     106,    0,  767,    2, 0x0a,  105 /* Public */,
     107,    0,  768,    2, 0x0a,  106 /* Public */,
     108,    0,  769,    2, 0x0a,  107 /* Public */,
     109,    0,  770,    2, 0x0a,  108 /* Public */,
     110,    0,  771,    2, 0x0a,  109 /* Public */,

 // signals: parameters
    QMetaType::Void, QMetaType::Int,    3,
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Int,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

Q_CONSTINIT const QMetaObject XxApp::staticMetaObject = { {
    QMetaObject::SuperData::link<QkApplication::staticMetaObject>(),
    qt_meta_stringdata_CLASSXxAppENDCLASS.offsetsAndSizes,
    qt_meta_data_CLASSXxAppENDCLASS,
    qt_static_metacall,
    nullptr,
    qt_incomplete_metaTypeArray<qt_meta_stringdata_CLASSXxAppENDCLASS_t,
        // Q_OBJECT / Q_GADGET
        QtPrivate::TypeAndForceComplete<XxApp, std::true_type>,
        // method 'cursorChanged'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        QtPrivate::TypeAndForceComplete<int, std::false_type>,
        // method 'textSizeChanged'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'updateWidgets'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'updateWidgetsForDiffs'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'onNbLinesChanged'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'openLeft'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'openMiddle'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'openRight'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'saveAsLeft'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'saveAsMiddle'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'saveAsRight'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'saveAsMerged'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'saveAs'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'generatePatchFromLeft'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'generatePatchFromMiddle'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'generatePatchFromRight'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'saveSelectedOnly'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'editLeft'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'editMiddle'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'editRight'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'saveOptions'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'quit'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'search'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'searchForward'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'searchBackward'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'redoDiff'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'editDiffOptions'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'editDisplayOptions'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'diffFilesAtCursor'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'nextAndDiffFiles'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'copyFileLeftToRight'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'copyFileRightToLeft'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'removeFileLeft'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'removeFileRight'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'nextDifference'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'previousDifference'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'nextUnselected'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'previousUnselected'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'selectGlobalLeft'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'selectGlobalMiddle'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'selectGlobalRight'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'selectGlobalNeither'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'selectGlobalUnselect'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'selectGlobalUnselectedLeft'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'selectGlobalUnselectedMiddle'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'selectGlobalUnselectedRight'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'selectGlobalUnselectedNeither'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'selectGlobalMerge'
        QtPrivate::TypeAndForceComplete<int, std::false_type>,
        // method 'selectRegionLeft'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'selectRegionMiddle'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'selectRegionRight'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'selectRegionNeither'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'selectRegionUnselect'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'selectRegionLeftAndNext'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'selectRegionMiddleAndNext'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'selectRegionRightAndNext'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'selectRegionNeitherAndNext'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'regionSplitSwapJoin'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'selectLineLeft'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'selectLineMiddle'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'selectLineRight'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'selectLineNeither'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'selectLineUnselect'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'tabsAt3'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'tabsAt4'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'tabsAt8'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'ignoreTrailing'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'ignoreWhitespace'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'ignoreCase'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'ignoreBlankLines'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'hideCarriageReturns'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'dirDiffRecursive'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'ignoreFileChanges'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'qualityNormal'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'qualityFastest'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'qualityHighest'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'toggleShowFilenames'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'togglePaneMergedView'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'togglePopupMergedView'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'toggleToolbar'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'toggleLineNumbers'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'toggleVerticalLine'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'toggleHorizNullMarkers'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'toggleOverview'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'hordiffTypeNone'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'hordiffTypeSingle'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'hordiffTypeMultiple'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'hordiffTypeUpdate'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'toggleIgnoreHorizontalWs'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'toggleIgnorePerHunkWs'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'toggleFormatClipboardText'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'ignoreFileNone'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'ignoreFileLeft'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'ignoreFileMiddle'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'ignoreFileRight'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'helpManPage'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'helpAbout'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'pageDown'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'pageUp'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'cursorDown'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'cursorUp'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'cursorTop'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'cursorBottom'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'quitAccept'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'quitReject'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'quitMerged'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'fontSizeIncrease'
        QtPrivate::TypeAndForceComplete<void, std::false_type>,
        // method 'fontSizeDecrease'
        QtPrivate::TypeAndForceComplete<void, std::false_type>
    >,
    nullptr
} };

void XxApp::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<XxApp *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->cursorChanged((*reinterpret_cast< std::add_pointer_t<int>>(_a[1]))); break;
        case 1: _t->textSizeChanged(); break;
        case 2: _t->updateWidgets(); break;
        case 3: _t->updateWidgetsForDiffs(); break;
        case 4: _t->onNbLinesChanged(); break;
        case 5: _t->openLeft(); break;
        case 6: _t->openMiddle(); break;
        case 7: _t->openRight(); break;
        case 8: _t->saveAsLeft(); break;
        case 9: _t->saveAsMiddle(); break;
        case 10: _t->saveAsRight(); break;
        case 11: _t->saveAsMerged(); break;
        case 12: _t->saveAs(); break;
        case 13: _t->generatePatchFromLeft(); break;
        case 14: _t->generatePatchFromMiddle(); break;
        case 15: _t->generatePatchFromRight(); break;
        case 16: _t->saveSelectedOnly(); break;
        case 17: _t->editLeft(); break;
        case 18: _t->editMiddle(); break;
        case 19: _t->editRight(); break;
        case 20: _t->saveOptions(); break;
        case 21: _t->quit(); break;
        case 22: _t->search(); break;
        case 23: _t->searchForward(); break;
        case 24: _t->searchBackward(); break;
        case 25: _t->redoDiff(); break;
        case 26: _t->editDiffOptions(); break;
        case 27: _t->editDisplayOptions(); break;
        case 28: _t->diffFilesAtCursor(); break;
        case 29: _t->nextAndDiffFiles(); break;
        case 30: _t->copyFileLeftToRight(); break;
        case 31: _t->copyFileRightToLeft(); break;
        case 32: _t->removeFileLeft(); break;
        case 33: _t->removeFileRight(); break;
        case 34: _t->nextDifference(); break;
        case 35: _t->previousDifference(); break;
        case 36: _t->nextUnselected(); break;
        case 37: _t->previousUnselected(); break;
        case 38: _t->selectGlobalLeft(); break;
        case 39: _t->selectGlobalMiddle(); break;
        case 40: _t->selectGlobalRight(); break;
        case 41: _t->selectGlobalNeither(); break;
        case 42: _t->selectGlobalUnselect(); break;
        case 43: _t->selectGlobalUnselectedLeft(); break;
        case 44: _t->selectGlobalUnselectedMiddle(); break;
        case 45: _t->selectGlobalUnselectedRight(); break;
        case 46: _t->selectGlobalUnselectedNeither(); break;
        case 47: { int _r = _t->selectGlobalMerge();
            if (_a[0]) *reinterpret_cast< int*>(_a[0]) = std::move(_r); }  break;
        case 48: _t->selectRegionLeft(); break;
        case 49: _t->selectRegionMiddle(); break;
        case 50: _t->selectRegionRight(); break;
        case 51: _t->selectRegionNeither(); break;
        case 52: _t->selectRegionUnselect(); break;
        case 53: _t->selectRegionLeftAndNext(); break;
        case 54: _t->selectRegionMiddleAndNext(); break;
        case 55: _t->selectRegionRightAndNext(); break;
        case 56: _t->selectRegionNeitherAndNext(); break;
        case 57: _t->regionSplitSwapJoin(); break;
        case 58: _t->selectLineLeft(); break;
        case 59: _t->selectLineMiddle(); break;
        case 60: _t->selectLineRight(); break;
        case 61: _t->selectLineNeither(); break;
        case 62: _t->selectLineUnselect(); break;
        case 63: _t->tabsAt3(); break;
        case 64: _t->tabsAt4(); break;
        case 65: _t->tabsAt8(); break;
        case 66: _t->ignoreTrailing(); break;
        case 67: _t->ignoreWhitespace(); break;
        case 68: _t->ignoreCase(); break;
        case 69: _t->ignoreBlankLines(); break;
        case 70: _t->hideCarriageReturns(); break;
        case 71: _t->dirDiffRecursive(); break;
        case 72: _t->ignoreFileChanges(); break;
        case 73: _t->qualityNormal(); break;
        case 74: _t->qualityFastest(); break;
        case 75: _t->qualityHighest(); break;
        case 76: _t->toggleShowFilenames(); break;
        case 77: _t->togglePaneMergedView(); break;
        case 78: _t->togglePopupMergedView(); break;
        case 79: _t->toggleToolbar(); break;
        case 80: _t->toggleLineNumbers(); break;
        case 81: _t->toggleVerticalLine(); break;
        case 82: _t->toggleHorizNullMarkers(); break;
        case 83: _t->toggleOverview(); break;
        case 84: _t->hordiffTypeNone(); break;
        case 85: _t->hordiffTypeSingle(); break;
        case 86: _t->hordiffTypeMultiple(); break;
        case 87: _t->hordiffTypeUpdate(); break;
        case 88: _t->toggleIgnoreHorizontalWs(); break;
        case 89: _t->toggleIgnorePerHunkWs(); break;
        case 90: _t->toggleFormatClipboardText(); break;
        case 91: _t->ignoreFileNone(); break;
        case 92: _t->ignoreFileLeft(); break;
        case 93: _t->ignoreFileMiddle(); break;
        case 94: _t->ignoreFileRight(); break;
        case 95: _t->helpManPage(); break;
        case 96: _t->helpAbout(); break;
        case 97: _t->pageDown(); break;
        case 98: _t->pageUp(); break;
        case 99: _t->cursorDown(); break;
        case 100: _t->cursorUp(); break;
        case 101: _t->cursorTop(); break;
        case 102: _t->cursorBottom(); break;
        case 103: _t->quitAccept(); break;
        case 104: _t->quitReject(); break;
        case 105: _t->quitMerged(); break;
        case 106: _t->fontSizeIncrease(); break;
        case 107: _t->fontSizeDecrease(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (XxApp::*)(int );
            if (_t _q_method = &XxApp::cursorChanged; *reinterpret_cast<_t *>(_a[1]) == _q_method) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (XxApp::*)();
            if (_t _q_method = &XxApp::textSizeChanged; *reinterpret_cast<_t *>(_a[1]) == _q_method) {
                *result = 1;
                return;
            }
        }
    }
}

const QMetaObject *XxApp::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *XxApp::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_CLASSXxAppENDCLASS.stringdata0))
        return static_cast<void*>(this);
    return QkApplication::qt_metacast(_clname);
}

int XxApp::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QkApplication::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 108)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 108;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 108)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 108;
    }
    return _id;
}

// SIGNAL 0
void XxApp::cursorChanged(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void XxApp::textSizeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}
QT_WARNING_POP
